@@ -9,7 +9,7 @@ import traceback
 from math import floor
 from pathlib import Path
 from threading import Thread
-from typing import List, Type, Set, Tuple
+from typing import List, Type, Set, Tuple, Optional
 
 import requests
 import yaml
@@ -17,7 +17,8 @@ from colorama import Fore
 from requests import exceptions, Response
 
 from bauh.api.abstract.context import ApplicationContext
-from bauh.api.abstract.controller import SoftwareManager, SearchResult, UpgradeRequirements, TransactionResult
+from bauh.api.abstract.controller import SoftwareManager, SearchResult, UpgradeRequirements, TransactionResult, \
+    SoftwareAction
 from bauh.api.abstract.disk import DiskCacheLoader
 from bauh.api.abstract.handler import ProcessWatcher, TaskManager
 from bauh.api.abstract.model import SoftwarePackage, CustomSoftwareAction, PackageSuggestion, PackageUpdate, \
@@ -26,16 +27,19 @@ from bauh.api.abstract.model import SoftwarePackage, CustomSoftwareAction, Packa
 from bauh.api.abstract.view import MessageType, MultipleSelectComponent, InputOption, SingleSelectComponent, \
     SelectViewType, TextInputComponent, FormComponent, FileChooserComponent, ViewComponent, PanelComponent
 from bauh.api.constants import DESKTOP_ENTRIES_DIR
-from bauh.commons import resource, user
-from bauh.commons.config import save_config
+from bauh.commons import resource
+from bauh.commons.boot import CreateConfigFile
 from bauh.commons.html import bold
 from bauh.commons.system import ProcessHandler, get_dir_size, get_human_size_str, SimpleProcess
 from bauh.gems.web import INSTALLED_PATH, nativefier, DESKTOP_ENTRY_PATH_PATTERN, URL_FIX_PATTERN, ENV_PATH, UA_CHROME, \
-    SEARCH_INDEX_FILE, SUGGESTIONS_CACHE_FILE, ROOT_DIR, CONFIG_FILE, TEMP_PATH, FIXES_PATH, ELECTRON_PATH
-from bauh.gems.web.config import read_config
+    SUGGESTIONS_CACHE_FILE, ROOT_DIR, TEMP_PATH, FIXES_PATH, ELECTRON_PATH, \
+    get_icon_path
+from bauh.gems.web.config import WebConfigManager
 from bauh.gems.web.environment import EnvironmentUpdater, EnvironmentComponent
 from bauh.gems.web.model import WebApplication
-from bauh.gems.web.worker import SuggestionsDownloader, SearchIndexGenerator
+from bauh.gems.web.search import SearchIndexManager
+from bauh.gems.web.worker import SuggestionsManager, UpdateEnvironmentSettings, \
+    SuggestionsLoader, SearchIndexGenerator
 
 try:
     from bs4 import BeautifulSoup, SoupStrainer
@@ -57,19 +61,20 @@ RE_SYMBOLS_SPLIT = re.compile(r'[\-|_\s:.]')
 
 class WebApplicationManager(SoftwareManager):
 
-    def __init__(self, context: ApplicationContext, suggestions_downloader: Thread = None):
+    def __init__(self, context: ApplicationContext, suggestions_loader: Optional[SuggestionsLoader] = None):
         super(WebApplicationManager, self).__init__(context=context)
         self.http_client = context.http_client
         self.env_updater = EnvironmentUpdater(logger=context.logger, http_client=context.http_client,
                                               file_downloader=context.file_downloader, i18n=context.i18n)
         self.enabled = True
         self.i18n = context.i18n
-        self.env_settings = {}
         self.logger = context.logger
         self.env_thread = None
-        self.suggestions_downloader = suggestions_downloader
+        self.suggestions_loader = suggestions_loader
         self.suggestions = {}
-        self.custom_actions = [CustomSoftwareAction(i18_label_key='web.custom_action.clean_env',
+        self.configman = WebConfigManager()
+        self.idxman = SearchIndexManager(logger=context.logger)
+        self.custom_actions = [CustomSoftwareAction(i18n_label_key='web.custom_action.clean_env',
                                                     i18n_status_key='web.custom_action.clean_env.status',
                                                     manager=self,
                                                     manager_method='clean_environment',
@@ -206,8 +211,8 @@ class WebApplicationManager(SoftwareManager):
             return BeautifulSoup(url_res.text, 'lxml', parse_only=SoupStrainer('head')), url_res
 
     def search(self, words: str, disk_loader: DiskCacheLoader, limit: int = -1, is_url: bool = False) -> SearchResult:
-        local_config = {}
-        thread_config = Thread(target=self._fill_config_async, args=(local_config,))
+        web_config = {}
+        thread_config = Thread(target=self._fill_config_async, args=(web_config,))
         thread_config.start()
 
         res = SearchResult([], [], 0)
@@ -240,8 +245,11 @@ class WebApplicationManager(SoftwareManager):
 
                     app = WebApplication(url=final_url, source_url=url, name=name, description=desc, icon_url=icon_url)
 
-                    if self.env_settings.get('electron') and self.env_settings['electron'].get('version'):
-                        app.version = self.env_settings['electron']['version']
+                    thread_config.join()
+                    env_settings = self.env_updater.read_settings(web_config=web_config, cache=True)
+
+                    if env_settings.get('electron') and env_settings['electron'].get('version'):
+                        app.version = env_settings['electron']['version']
                         app.latest_version = app.version
 
                     res.new = [app]
@@ -249,11 +257,11 @@ class WebApplicationManager(SoftwareManager):
             lower_words = words.lower().strip()
             installed_matches = [app for app in installed if lower_words in app.name.lower()]
 
-            index = self._read_search_index()
+            index = self.idxman.read()
 
-            if not index and self.suggestions_downloader and self.suggestions_downloader.is_alive():
-                self.suggestions_downloader.join()
-                index = self._read_search_index()
+            if not index and self.suggestions_loader and self.suggestions_loader.is_alive():
+                self.suggestions_loader.join()
+                index = self.idxman.read()
 
             if index:
                 split_words = lower_words.split(' ')
@@ -294,6 +302,9 @@ class WebApplicationManager(SoftwareManager):
                             else:
                                 matched_suggestions.sort(key=lambda s: s.get('priority', 0), reverse=True)
 
+                                thread_config.join()
+                                env_settings = self.env_updater.read_settings(web_config=web_config, cache=True)
+
                                 if installed_matches:
                                     # checking if any of the installed matches is one of the matched suggestions
 
@@ -305,11 +316,11 @@ class WebApplicationManager(SoftwareManager):
                                         if found:
                                             res.installed.extend(found)
                                         else:
-                                            res.new.append(self._map_suggestion(sug).package)
+                                            res.new.append(self._map_suggestion(sug, env_settings).package)
 
                                 else:
                                     for sug in matched_suggestions:
-                                        res.new.append(self._map_suggestion(sug).package)
+                                        res.new.append(self._map_suggestion(sug, env_settings).package)
 
         res.total += len(res.installed)
         res.total += len(res.new)
@@ -317,21 +328,14 @@ class WebApplicationManager(SoftwareManager):
         if res.new:
             thread_config.join()
 
-            if local_config['environment']['electron']['version']:
+            if web_config['environment']['electron']['version']:
                 for app in res.new:
-                    app.version = str(local_config['environment']['electron']['version'])
+                    app.version = str(web_config['environment']['electron']['version'])
                     app.latest_version = app.version
 
         return res
 
-    def _read_search_index(self) -> dict:
-        if os.path.exists(SEARCH_INDEX_FILE):
-            with open(SEARCH_INDEX_FILE) as f:
-                return yaml.safe_load(f.read())
-        else:
-            self.logger.warning("No search index found at {}".format(SEARCH_INDEX_FILE))
-
-    def read_installed(self, disk_loader: DiskCacheLoader, limit: int = -1, only_apps: bool = False, pkg_types: Set[Type[SoftwarePackage]] = None, internet_available: bool = True) -> SearchResult:
+    def read_installed(self, disk_loader: Optional[DiskCacheLoader], limit: int = -1, only_apps: bool = False, pkg_types: Set[Type[SoftwarePackage]] = None, internet_available: bool = True) -> SearchResult:
         res = SearchResult([], [], 0)
 
         if os.path.exists(INSTALLED_PATH):
@@ -461,7 +465,7 @@ class WebApplicationManager(SoftwareManager):
                     break
 
         inp_cat = SingleSelectComponent(label=self.i18n['category'], type_=SelectViewType.COMBO, options=cat_ops, default_option=def_cat)
-
+        op_wv = InputOption(id_='widevine', label=self.i18n['web.install.option.widevine.label'] + ' (DRM)', value="--widevine", tooltip=self.i18n['web.install.option.widevine.tip'])
         tray_op_off = InputOption(id_='tray_off', label=self.i18n['web.install.option.tray.off.label'], value=0, tooltip=self.i18n['web.install.option.tray.off.tip'])
         tray_op_default = InputOption(id_='tray_def', label=self.i18n['web.install.option.tray.default.label'], value='--tray', tooltip=self.i18n['web.install.option.tray.default.tip'])
         tray_op_min = InputOption(id_='tray_min', label=self.i18n['web.install.option.tray.min.label'], value='--tray=start-in-tray', tooltip=self.i18n['web.install.option.tray.min.tip'])
@@ -503,7 +507,7 @@ class WebApplicationManager(SoftwareManager):
         op_insecure = InputOption(id_='insecure', label=self.i18n['web.install.option.insecure.label'], value="--insecure", tooltip=self.i18n['web.install.option.insecure.tip'])
         op_igcert = InputOption(id_='ignore_certs', label=self.i18n['web.install.option.ignore_certificate.label'], value="--ignore-certificate", tooltip=self.i18n['web.install.option.ignore_certificate.tip'])
 
-        adv_opts = [op_single, op_allow_urls, op_max, op_fs, op_nframe, op_ncache, op_insecure, op_igcert]
+        adv_opts = [op_single, op_allow_urls, op_wv, op_max, op_fs, op_nframe, op_ncache, op_insecure, op_igcert]
         def_adv_opts = {op_single, op_allow_urls}
 
         if app.preset_options:
@@ -619,8 +623,8 @@ class WebApplicationManager(SoftwareManager):
             traceback.print_exc()
 
     def install(self, pkg: WebApplication, root_password: str, disk_loader: DiskCacheLoader, watcher: ProcessWatcher) -> TransactionResult:
-
         continue_install, install_options = self._ask_install_options(pkg, watcher)
+        widevine_support = '--widevine' in install_options
 
         if not continue_install:
             return TransactionResult(success=False, installed=[], removed=[])
@@ -628,16 +632,18 @@ class WebApplicationManager(SoftwareManager):
         watcher.change_substatus(self.i18n['web.env.checking'])
         handler = ProcessHandler(watcher)
 
-        env_settings = self.env_updater.read_settings()
-        local_config = read_config()
+        web_config = self.configman.get_config()
+        env_settings = self.env_updater.read_settings(web_config=web_config)
 
-        if local_config['environment']['system'] and not nativefier.is_available():
+        if web_config['environment']['system'] and not nativefier.is_available():
             watcher.show_message(title=self.i18n['error'].capitalize(),
                                  body=self.i18n['web.install.global_nativefier.unavailable'].format(n=bold('Nativefier'), app=bold(pkg.name)) + '.',
                                  type_=MessageType.ERROR)
             return TransactionResult(success=False, installed=[], removed=[])
 
-        env_components = self.env_updater.check_environment(app=pkg, local_config=local_config, env=env_settings, is_x86_x64_arch=self.context.is_system_x86_64())
+        env_components = self.env_updater.check_environment(app=pkg, local_config=web_config, env=env_settings,
+                                                            is_x86_x64_arch=self.context.is_system_x86_64(),
+                                                            widevine=widevine_support)
 
         comps_to_update = [c for c in env_components if c.update]
 
@@ -683,6 +689,9 @@ class WebApplicationManager(SoftwareManager):
                 install_options.append('--icon={}'.format(temp_icon_path))
 
                 self.logger.info("Writing a temp suggestion icon at {}".format(temp_icon_path))
+
+                Path(os.path.dirname(temp_icon_path)).mkdir(parents=True, exist_ok=True)
+
                 with open(temp_icon_path, 'wb+') as f:
                     f.write(icon_bytes)
 
@@ -690,8 +699,8 @@ class WebApplicationManager(SoftwareManager):
 
         electron_version = str(next((c for c in env_components if c.id == 'electron')).version)
         installed = handler.handle_simple(nativefier.install(url=pkg.url, name=app_id, output_dir=app_dir,
-                                                             electron_version=electron_version,
-                                                             system=bool(local_config['environment']['system']),
+                                                             electron_version=electron_version if not widevine_support else None,
+                                                             system=bool(web_config['environment']['system']),
                                                              cwd=INSTALLED_PATH,
                                                              extra_options=install_options))
 
@@ -769,7 +778,7 @@ class WebApplicationManager(SoftwareManager):
         return """
         [Desktop Entry]
         Type=Application
-        Name={name} ( web )
+        Name={name} (web)
         Comment={desc}
         Icon={icon}
         Exec={exec_path}
@@ -788,7 +797,7 @@ class WebApplicationManager(SoftwareManager):
 
     def can_work(self) -> bool:
         if BS4_AVAILABLE and LXML_AVAILABLE:
-            config = read_config(update_file=True)
+            config = self.configman.get_config()
             use_system_env = config['environment']['system']
 
             if not use_system_env:
@@ -798,27 +807,43 @@ class WebApplicationManager(SoftwareManager):
 
         return False
 
-    def requires_root(self, action: str, pkg: SoftwarePackage):
+    def requires_root(self, action: SoftwareAction, pkg: SoftwarePackage) -> bool:
         return False
 
-    def _update_env_settings(self, task_manager: TaskManager = None):
-        self.env_settings = self.env_updater.read_settings(task_manager)
-
-    def _download_suggestions(self, taskman: TaskManager = None):
-        downloader = SuggestionsDownloader(logger=self.logger, http_client=self.http_client, i18n=self.i18n, taskman=taskman)
-        self.suggestions = downloader.download()
-
-        if self.suggestions:
-            index_gen = SearchIndexGenerator(logger=self.logger)
-            Thread(target=index_gen.generate_index, args=(self.suggestions,), daemon=True).start()
+    def _assign_suggestions(self, suggestions: dict):
+        self.suggestions = suggestions
 
     def prepare(self, task_manager: TaskManager, root_password: str, internet_available: bool):
-        if internet_available:
-            self.env_thread = Thread(target=self._update_env_settings, args=(task_manager,), daemon=True)
-            self.env_thread.start()
+        create_config = CreateConfigFile(taskman=task_manager, configman=self.configman, i18n=self.i18n,
+                                         task_icon_path=get_icon_path(), logger=self.logger)
+        create_config.start()
 
-            self.suggestions_downloader = Thread(target=self._download_suggestions, args=(task_manager,), daemon=True)
-            self.suggestions_downloader.start()
+        if internet_available:
+            UpdateEnvironmentSettings(env_updater=EnvironmentUpdater(logger=self.context.logger,
+                                                                     i18n=self.i18n,
+                                                                     http_client=self.http_client,
+                                                                     file_downloader=self.context.file_downloader,
+                                                                     taskman=task_manager),
+                                      taskman=task_manager,
+                                      create_config=create_config,
+                                      i18n=self.i18n).start()
+
+        self.suggestions_loader = SuggestionsLoader(manager=SuggestionsManager(logger=self.logger,
+                                                                               http_client=self.http_client,
+                                                                               i18n=self.i18n),
+                                                    logger=self.logger,
+                                                    i18n=self.i18n,
+                                                    taskman=task_manager,
+                                                    suggestions_callback=self._assign_suggestions,
+                                                    create_config=create_config,
+                                                    internet_connection=internet_available)
+        self.suggestions_loader.start()
+
+        SearchIndexGenerator(taskman=task_manager,
+                             idxman=self.idxman,
+                             suggestions_loader=self.suggestions_loader,
+                             i18n=self.i18n,
+                             logger=self.logger).start()
 
     def list_updates(self, internet_available: bool) -> List[PackageUpdate]:
         pass
@@ -853,7 +878,7 @@ class WebApplicationManager(SoftwareManager):
 
         app.status = PackageStatus.READY
 
-    def _map_suggestion(self, suggestion: dict) -> PackageSuggestion:
+    def _map_suggestion(self, suggestion: dict, env_settings: Optional[dict]) -> PackageSuggestion:
         app = WebApplication(name=suggestion.get('name'),
                              url=suggestion.get('url'),
                              icon_url=suggestion.get('icon_url'),
@@ -870,8 +895,8 @@ class WebApplicationManager(SoftwareManager):
         elif isinstance(description, str):
             app.description = description
 
-        if not app.version and self.env_settings and self.env_settings.get('electron'):
-            app.version = self.env_settings['electron']['version']
+        if not app.version and env_settings and env_settings.get('electron'):
+            app.version = env_settings['electron']['version']
             app.latest_version = app.version
 
         app.status = PackageStatus.LOADING_DATA
@@ -881,24 +906,24 @@ class WebApplicationManager(SoftwareManager):
         return PackageSuggestion(priority=SuggestionPriority(suggestion['priority']), package=app)
 
     def _fill_config_async(self, output: dict):
-        output.update(read_config())
+        output.update(self.configman.get_config())
 
     def list_suggestions(self, limit: int, filter_installed: bool) -> List[PackageSuggestion]:
-        local_config = {}
+        web_config = {}
 
-        thread_config = Thread(target=self._fill_config_async, args=(local_config,))
+        thread_config = Thread(target=self._fill_config_async, args=(web_config,))
         thread_config.start()
 
         if self.suggestions:
             suggestions = self.suggestions
-        elif self.suggestions_downloader:
-            self.suggestions_downloader.join(5)
+        elif self.suggestions_loader:
+            self.suggestions_loader.join(5)
             suggestions = self.suggestions
         else:
-            suggestions = SuggestionsDownloader(logger=self.logger, http_client=self.http_client, i18n=self.i18n).download()
+            suggestions = SuggestionsManager(logger=self.logger, http_client=self.http_client, i18n=self.i18n).download()
 
         # cleaning memory
-        self.suggestions_downloader = None
+        self.suggestions_loader = None
         self.suggestions = None
 
         if suggestions:
@@ -910,7 +935,7 @@ class WebApplicationManager(SoftwareManager):
             else:
                 installed = None
 
-            res = []
+            env_settings, res = None, []
 
             for s in suggestion_list:
                 if limit <= 0 or len(res) < limit:
@@ -920,24 +945,24 @@ class WebApplicationManager(SoftwareManager):
                         if surl in installed:
                             continue
 
-                    res.append(self._map_suggestion(s))
+                    if env_settings is None:  # reading settings if not already loaded
+                        thread_config.join()
+                        env_settings = self.env_updater.read_settings(web_config=web_config)
+
+                    res.append(self._map_suggestion(s, env_settings))
                 else:
                     break
 
             if res:
-                if not self.env_settings and self.env_thread:
-                    self.env_thread.join()
-                    self.env_thread = None  # cleaning memory
-
-                if self.env_settings:
+                if env_settings:
                     for s in res:
-                        s.package.version = self.env_settings['electron']['version']
+                        s.package.version = env_settings['electron']['version']
                         s.package.latest_version = s.package.version
 
                 thread_config.join()
-                if local_config and local_config['environment']['electron']['version']:
+                if web_config and web_config['environment']['electron']['version']:
                     for s in res:
-                        s.package.version = str(local_config['environment']['electron']['version'])
+                        s.package.version = str(web_config['environment']['electron']['version'])
                         s.package.latest_version = s.package.version
 
             return res
@@ -949,7 +974,7 @@ class WebApplicationManager(SoftwareManager):
         return True
 
     def launch(self, pkg: WebApplication):
-        subprocess.Popen(pkg.get_command(), shell=user.is_root())
+        subprocess.Popen(args=[pkg.get_command()], shell=True, env={**os.environ})
 
     def get_screenshots(self, pkg: SoftwarePackage) -> List[str]:
         pass
@@ -969,11 +994,11 @@ class WebApplicationManager(SoftwareManager):
                     traceback.print_exc()
 
     def get_settings(self, screen_width: int, screen_height: int) -> ViewComponent:
-        config = read_config()
+        web_config = self.configman.get_config()
         max_width = floor(screen_width * 0.15)
 
         input_electron = TextInputComponent(label=self.i18n['web.settings.electron.version.label'],
-                                            value=config['environment']['electron']['version'],
+                                            value=web_config['environment']['electron']['version'],
                                             tooltip=self.i18n['web.settings.electron.version.tooltip'],
                                             placeholder='{}: 7.1.0'.format(self.i18n['example.short']),
                                             max_width=max_width,
@@ -986,35 +1011,55 @@ class WebApplicationManager(SoftwareManager):
 
         select_nativefier = SingleSelectComponent(label="Nativefier",
                                                   options=native_opts,
-                                                  default_option=[o for o in native_opts if o.value == config['environment']['system']][0],
+                                                  default_option=[o for o in native_opts if o.value == web_config['environment']['system']][0],
                                                   type_=SelectViewType.COMBO,
                                                   tooltip=self.i18n['web.settings.nativefier.tip'],
                                                   max_width=max_width,
                                                   id_='nativefier')
 
-        form_env = FormComponent(label=self.i18n['web.settings.nativefier.env'].capitalize(), components=[input_electron, select_nativefier])
+        env_settings_exp = TextInputComponent(label=self.i18n['web.settings.cache_exp'],
+                                              tooltip=self.i18n['web.settings.cache_exp.tip'],
+                                              capitalize_label=False,
+                                              value=int(web_config['environment']['cache_exp']) if isinstance(web_config['environment']['cache_exp'], int) else '',
+                                              only_int=True,
+                                              max_width=max_width,
+                                              id_='web_cache_exp')
+
+        sugs_exp = TextInputComponent(label=self.i18n['web.settings.suggestions.cache_exp'],
+                                      tooltip=self.i18n['web.settings.suggestions.cache_exp.tip'],
+                                      capitalize_label=False,
+                                      value=int(web_config['suggestions']['cache_exp']) if isinstance(
+                                          web_config['suggestions']['cache_exp'], int) else '',
+                                      only_int=True,
+                                      max_width=max_width,
+                                      id_='web_sugs_exp')
+
+        form_env = FormComponent(label=self.i18n['web.settings.nativefier.env'].capitalize(),
+                                 components=[input_electron, select_nativefier, env_settings_exp, sugs_exp])
 
         return PanelComponent([form_env])
 
-    def save_settings(self, component: PanelComponent) -> Tuple[bool, List[str]]:
-        config = read_config()
+    def save_settings(self, component: PanelComponent) -> Tuple[bool, Optional[List[str]]]:
+        web_config = self.configman.get_config()
 
         form_env = component.components[0]
 
-        config['environment']['electron']['version'] = str(form_env.get_component('electron_version').get_value()).strip()
+        web_config['environment']['electron']['version'] = str(form_env.get_component('electron_version').get_value()).strip()
 
-        if len(config['environment']['electron']['version']) == 0:
-            config['environment']['electron']['version'] = None
+        if len(web_config['environment']['electron']['version']) == 0:
+            web_config['environment']['electron']['version'] = None
 
         system_nativefier = form_env.get_component('nativefier').get_selected()
 
         if system_nativefier and not nativefier.is_available():
             return False, [self.i18n['web.settings.env.nativefier.system.not_installed'].format('Nativefier')]
 
-        config['environment']['system'] = system_nativefier
+        web_config['environment']['system'] = system_nativefier
+        web_config['environment']['cache_exp'] = form_env.get_component('web_cache_exp').get_int_value()
+        web_config['suggestions']['cache_exp'] = form_env.get_component('web_sugs_exp').get_int_value()
 
         try:
-            save_config(config, CONFIG_FILE)
+            self.configman.save_config(web_config)
             return True, None
         except:
             return False, [traceback.format_exc()]
